//
// file :         rampingthread.cpp
//
// description :  ramping thread
//
// project :      TANGO Device Server
//
// $Author:
//
// $Revision:
//
// $Log: 
//
// copyleft :   Sincrotrone Trieste S.C.p.A.
//              Strada Statale 14 - km 163,5 in AREA Science Park
//              34012 Basovizza, Trieste ITALY

#include "util.h"
#include "Sequencer.h"


/* 
 *	Sequencer::Sequencer()
 *	Class constructor; instantiate the device proxy and initialize some variables 
 * 	// configuration syntax: device name
 *                           timeout (seconds)
 * If sequence not initialized: NONE;0
 */		
Sequencer::Sequencer(std::string &configuration) 
{

	if (configuration == "NONE") {
		device_name = configuration;
		init_flag = false;
		timeout = 0;
		state = Tango::DISABLE;
		return;
	}

	// fill class members
	device_name = configuration;


	safe_mode = true;
	mutex = new omni_mutex();
	wait_flag = false;
	init_flag = true;
	state = Tango::OFF;

	log_hist.set_size(MIMOFB_LOG_SIZE);
	log_hist.set_log_level(MIMOFB_LOGLEVEL_INFO);

	// connect to device and throw an exception if it fails
	device = 0;

	device = new Tango::DeviceProxy(device_name);
	int32_t default_timeout = 1000;
	device->set_timeout_millis(default_timeout);
	device->set_source(Tango::DEV);

}


Sequencer::~Sequencer(void) 
{


	if (!init_flag)
		return;

	mutex->lock();

	if (device)
		delete device;

	mutex->unlock();

	delete mutex;

}


Sequencer::Sequencer(const Sequencer &sequencer_obj)
{

	state = sequencer_obj.state;
	device_name = sequencer_obj.device_name;
	timeout = sequencer_obj.timeout;
	wait_flag = sequencer_obj.wait_flag;
	init_flag = sequencer_obj.init_flag;
	log_hist = sequencer_obj.log_hist;
	start_time.tv_sec = sequencer_obj.start_time.tv_sec;
	start_time.tv_usec = sequencer_obj.start_time.tv_usec;


	if (state == Tango::DISABLE) {
		return;
	}

	mutex = new omni_mutex();
	device = 0;
	device = new Tango::DeviceProxy(device_name);
	device->set_source(Tango::DEV);
	int32_t default_timeout = 1000;
	device->set_timeout_millis(default_timeout);


}


void Sequencer::start(void)
{

	if (!init_flag)
		return;

	mutex->lock();

	try {
		device->command_inout("Acknowledge");
	}
	catch (Tango::DevFailed &e) {}

	try {
		device->command_inout("Start");
		mutex->unlock();
	}
	catch (Tango::DevFailed &e) {
		string tmp_str(e.errors[0].desc);
		state = Tango::FAULT;
		log_hist.add_log(tmp_str.c_str(), MIMOFB_LOGLEVEL_DEBUG);
		mutex->unlock();
		Tango::Except::re_throw_exception(e,
			(const char *) "Failed to start sequence",
			(const char *) "Failed to start sequence",
		__FUNCTION__, Tango::ERR);
	}
	catch(...) {
		string tmp_str("Generic exception");
		state = Tango::FAULT;
		log_hist.add_log(tmp_str.c_str(), MIMOFB_LOGLEVEL_DEBUG);
		mutex->unlock();
		Tango::Except::throw_exception(
			(const char *) "Failed to start sequence",
			(const char *) "Generic exception",
		__FUNCTION__, Tango::ERR);
	}

	gettimeofday(&start_time, NULL);
}


Tango::DevState Sequencer::wait(bool blocking)
{
	struct timeval now;

	bool read_once = false;

	if (!init_flag) {
		
		return Tango::DISABLE;
	}

	if (blocking) 
		wait_flag = true;
	else 
		wait_flag = false;
		
	while (wait_flag) {

		// wait  100ms 
		for (int i = 0; i < 10; i++) {
			usleep(10000);
			if (!wait_flag)
				break;
		}

		mutex->lock();
		try {
			device->set_source(Tango::DEV);
			device->read_attribute("State") >> state;
			// get the sequencer timeout
			if (read_once == false) {
				read_once = true;
				long elapsed_time = 0, remaining_time = 0;
				device->read_attribute("ElapsedTime") >> elapsed_time;
				device->read_attribute("RemainingTime") >> remaining_time;
				timeout = elapsed_time + remaining_time + 60; // 30 sec of margin
			}
		}
		catch (Tango::DevFailed &e) {
			string tmp_str(e.errors[0].desc);
			if (state != Tango::FAULT)
				log_hist.add_log(tmp_str.c_str(), MIMOFB_LOGLEVEL_DEBUG);
			state = Tango::FAULT;
			mutex->unlock();
			Tango::Except::re_throw_exception(e,
				(const char *) "Failed reading sequence state/timeout",
				(const char *) tmp_str.c_str(),
				__FUNCTION__, Tango::ERR);
		}
		catch(...) {
			string tmp_str("Generic exception");
			if (state != Tango::FAULT)
				log_hist.add_log(tmp_str.c_str(), MIMOFB_LOGLEVEL_DEBUG);
			state = Tango::FAULT;
			mutex->unlock();
			Tango::Except::throw_exception(
				(const char *) "Failed waiting sequence to complete",
				(const char *) "Generic exception",
			__FUNCTION__, Tango::ERR);
		}
		mutex->unlock();


		if (state == Tango::OFF) {
			break;
		}
		if (state == Tango::FAULT) {
			string step_status, step_error;
			try {
				device->read_attribute("StepStatus") >> step_status;
				device->read_attribute("StepError") >> step_error;
			}
			catch (Tango::DevFailed &e) {
				string tmp_str(e.errors[0].desc);
				if (state != Tango::FAULT)
					log_hist.add_log(tmp_str.c_str(), MIMOFB_LOGLEVEL_DEBUG);
				state = Tango::FAULT;
				mutex->unlock();
				Tango::Except::re_throw_exception(e,
					(const char *) "Failed reading sequence state/timeout",
					(const char *) tmp_str.c_str(),
					__FUNCTION__, Tango::ERR);
			}
			catch(...) {
				string tmp_str("Generic exception");
				if (state != Tango::FAULT)
					log_hist.add_log(tmp_str.c_str(), MIMOFB_LOGLEVEL_DEBUG);
				state = Tango::FAULT;
				mutex->unlock();
				Tango::Except::throw_exception(
					(const char *) "Failed waiting sequence to complete",
					(const char *) "Generic exception",
				__FUNCTION__, Tango::ERR);
			}
			string err_str;
			err_str.append(step_status);err_str.append(",");err_str.append(step_error);
			log_hist.add_log(err_str.c_str(), MIMOFB_LOGLEVEL_DEBUG);
			mutex->unlock();
			Tango::Except::throw_exception(
				(const char *) "Sequence failure",
				(const char *) step_status.c_str(),
				__FUNCTION__, Tango::ERR);
		}

		gettimeofday(&now, NULL);
		int32_t tim = (now.tv_sec - start_time.tv_sec);
		if (tim > timeout) {
			TangoSys_MemStream error_stream;
			error_stream << "Sequence timeout (wait=" << tim << ", timeout=" <<  timeout << ") " << ends;
			string tmp_str(error_stream.str());
			if (state != Tango::FAULT)
				log_hist.add_log(tmp_str.c_str(), MIMOFB_LOGLEVEL_DEBUG);
			state = Tango::FAULT;
			Tango::Except::throw_exception(
				(const char *)"Timeout expired",
				error_stream.str(),
				__FUNCTION__, Tango::ERR);
		}

	}

	return state;

}
	



void Sequencer::stop(void)
{

	if (!init_flag)
		return;
	wait_flag = false;

	mutex->lock();
	try {
		device->command_inout("Stop");
	}
	catch (Tango::DevFailed &e) {
		state = Tango::FAULT;
		string tmp_str(e.errors[0].desc);
		log_hist.add_log(tmp_str.c_str(), MIMOFB_LOGLEVEL_DEBUG);
		mutex->unlock();
		Tango::Except::re_throw_exception(e,
			(const char *) "Failed stopping sequence",
			(const char *) "Failed stopping sequence",
			__FUNCTION__, Tango::ERR);
	}
	catch(...) {
		state = Tango::FAULT;
		string tmp_str("Generic exception");
		log_hist.add_log(tmp_str.c_str(), MIMOFB_LOGLEVEL_DEBUG);
		mutex->unlock();
		Tango::Except::throw_exception(
			(const char *) "Failed stopping sequence",
			(const char *) "Generic exception",
			__FUNCTION__, Tango::ERR);
	}
	mutex->unlock();

}



Tango::DevState Sequencer::get_state(void)
{

	if (!init_flag)
		return Tango::DISABLE;

	return state;

}


vector<string> Sequencer::get_log_hist(int32_t num_values)
{
	
	return log_hist.get_log_hist(num_values, false); //LIFO

}


void Sequencer::set_log_level(int32_t level) 
{

	if ((level < MIMOFB_LOGLEVEL_NONE) || (level > MIMOFB_LOGLEVEL_DEBUG)) {
		Tango::Except::throw_exception(
			(const char *) "Failed to set log level",
			(const char *) "Invalid log level value (0=off,1=basic=,2=debug)",
			__FUNCTION__, Tango::ERR);
	}

	log_hist.set_log_level(level);

}

int32_t Sequencer::get_log_level(void) 
{

	return log_hist.get_log_level();

}


string Sequencer::get_label(void) 
{

	return device_name;

}




