#include "util.h"


/*
 * function which split string (once defined a separator 
 */
void string_explode(string str, string separator, vector<string>* results)
{
	size_t found;

	found = str.find_first_of(separator);
	while(found != string::npos) {
		if(found > 0) {
			results->push_back(str.substr(0,found));
		}
		str = str.substr(found+1);
		found = str.find_first_of(separator);
	}
	if(str.length() > 0) {
		results->push_back(str);
	}

}

/*
 * sleep time in msec
 */
int local_msleep(long msec)
{
	struct timeval tv;

	tv.tv_sec = msec / 1000L;
	tv.tv_usec = (msec - tv.tv_sec*1000) * 1000;

	return select(0, 0, 0, 0, &tv);

}


/**
 * Generate a disorderd sequence of n-1 unique integer numbers between 0 and n-1
 */
vector<int32_t> shuffled_vector_index(int32_t num_elem)
{
	vector<int32_t> myvector;

	for (int32_t i = 0; i < num_elem; i++) 
		myvector.push_back(i);
	
	srand ( time(NULL) );
	random_shuffle( myvector.begin(), myvector.end() );

	return myvector;

}


/*
 * generate a random uniform distributed number between 0 and 1
 */
double random_uniform_generator()
{
  return ( (double)(rand()) + 1. )/( (double)(RAND_MAX) + 1. );
}


/*
 * generate a gaussian distributed number with mean 0 and std equal to 1 --- MEAN + STD*random_gaussian_generator()
 */
double random_gaussian_generator()
{
  double x = random_uniform_generator();
  double y = random_uniform_generator();
  return cos(2*M_PI*x)*sqrt(-2.*log(y));
}


/*
 * process_data
 */
double low_pass_filt_data(vector <double> values, int32_t process_mode) 
{

	if (values.size() == 0) {
		Tango::Except::throw_exception(
			(const char *) "Values out of range",
			(const char *) "Empty data",
			__FUNCTION__, Tango::ERR);		
	}

	if ((process_mode == MIMOFB_FILT_NONE) || (values.size() == 1)) {
		return values[0];
	}
	else if (process_mode == MIMOFB_FILT_MEDIAN) {
		if (values.size() > 1) {
			size_t n = values.size() / 2;
			sort(values.begin(), values.end());
			nth_element(values.begin(), values.begin()+n, values.end());
			return values[n];
		}
		else {
			return values[0];
		}
	}
	else if (process_mode == MIMOFB_FILT_MEAN) {
		return accumulate(values.begin(), values.end(), 0.0) / values.size(); 
	}
	else if (process_mode == MIMOFB_FILT_STD) {
		if (values.size() > 1) {
			return std_dev(values);
		}
		else {
			return 0;
		}
	}
	else {
		Tango::Except::throw_exception(
			(const char *) "Failed to process data",
			(const char *) "Invalid process mode (0=none,1=mean,2=median,3=std)",
			__FUNCTION__, Tango::ERR);
	}

	// never reach here
	return 0;

}

/*
 * check_limits
 */
void check_limits(double value, double min_value, double max_value)
{
	if ((value < min_value) || (value > max_value)) {
		TangoSys_MemStream error_stream;
		error_stream << "Out of range value (min=" << min_value << ", max=" << max_value << ",val=" << value << ")";
		Tango::Except::throw_exception(
			(const char *) "Data out of range",
			error_stream.str(),
			__FUNCTION__, Tango::ERR);
	}
}

/*
 * out_of_range_filt_data
 */
vector <double> out_of_range_filt_data(vector <double> values, double min_value, double max_value, double tol, bool error_zero, bool skip_zero)
{
	int cnt = 0;
	vector <double> inrange_values;

	for (std::vector<double>::iterator it = values.begin() ; it != values.end(); ++it) {
		if ((*it < min_value) || (*it > max_value) || ((*it == 0) && error_zero) || std::isinf(*it) || std::isnan(*it)) {
			cnt++;
		}
		else {
			// do not save zero values if skip_zero == true
			if ((skip_zero == false) || (*it != 0))
				inrange_values.push_back(*it);
		}
	}
	
	if (values.size() == 0) {
		TangoSys_MemStream error_stream;
		error_stream << "Data empty (" << cnt << "/" << values.size() << ")";
		Tango::Except::throw_exception(
			(const char *) "Data out of range",
			error_stream.str(),
			__FUNCTION__, Tango::ERR);
	}

	if (((double)cnt / (double)values.size()) > tol) {
		TangoSys_MemStream error_stream;
		error_stream << "Too many out of range samples (" << cnt << "/" << values.size() << ")";
		Tango::Except::throw_exception(
			(const char *) "Data out of range",
			error_stream.str(),
			__FUNCTION__, Tango::ERR);
	}
	
	return inrange_values;
	
}

/*
 * check tango states
 */
Tango::DevState check_state(vector <Tango::DevState> &allowed_states, Tango::DeviceProxy *device, long max_retry, long retry_sleep)
{
	Tango::DevState curr_state;
	TangoSys_MemStream error_stream;

	for (int i = 0; i <= max_retry; i++) {
		device->read_attribute("State") >> curr_state;

		for (std::vector<Tango::DevState>::iterator it = allowed_states.begin() ; it != allowed_states.end(); ++it) {
			if (*it == curr_state)
				return curr_state;
		}

		// do a retry only when in moving or running states
		if ((curr_state == Tango::MOVING) || (curr_state == Tango::RUNNING)) {
			local_msleep(retry_sleep);
		}
		// when in fault state half the number of retries
		else if (curr_state == Tango::FAULT) {
			i = i+1;
			local_msleep(retry_sleep);
		}
		else {
			break;
		}

	}

	error_stream << "Not allowed state (" << curr_state << ")"; 

	// if no allowed state found return exception
	Tango::Except::throw_exception(
		(const char *)"Not allowed state",
		error_stream.str(),
		__FUNCTION__, Tango::ERR);
}


void cmd_extract_values(Tango::DeviceData *cmd_value, vector<double> &val, vector<string> &val_string)
{

	Tango::DevState stval;
	vector<Tango::DevState> v_st;
	vector<Tango::DevULong> v_ulo;
	vector<Tango::DevUChar> v_uch;
	vector<Tango::DevShort> v_sh;
	vector<Tango::DevUShort> v_ush;
	vector<Tango::DevLong> v_lo;
	vector<Tango::DevDouble> v_do;
	vector<Tango::DevFloat> v_fl;
	vector<Tango::DevBoolean> v_bo;
	vector<Tango::DevLong64> v_lo64;
	vector<Tango::DevULong64> v_ulo64;
	//vector<string> v_string;

	long cmd_type = cmd_value->get_type();

	val.clear();

	if (cmd_type == Tango::DEV_UCHAR)
	{
		*cmd_value >> v_uch;
		for(vector<Tango::DevUChar>::iterator it = v_uch.begin(); it != v_uch.end(); it++)
			val.push_back((double)(*it));		//convert all to double
	}
	else if ((cmd_type == Tango::DEV_SHORT) || (cmd_type == Tango::DEVVAR_SHORTARRAY)) 
	{
		*cmd_value >> v_sh;
		for(vector<Tango::DevShort>::iterator  it = v_sh.begin(); it != v_sh.end(); it++)
			val.push_back((double)(*it));		//convert all to double
	}
	else if ((cmd_type == Tango::DEV_USHORT) || (cmd_type == Tango::DEVVAR_USHORTARRAY)) 
	{
		*cmd_value >> v_ush;
		for(vector<Tango::DevUShort>::iterator  it = v_ush.begin(); it != v_ush.end(); it++)
			val.push_back((double)(*it));		//convert all to double
	}
	else if ((cmd_type == Tango::DEV_LONG) || (cmd_type == Tango::DEVVAR_LONGARRAY)) 
	{
		*cmd_value >> v_lo;
		for(vector<Tango::DevLong>::iterator  it = v_lo.begin(); it != v_lo.end(); it++)
			val.push_back((double)(*it));		//convert all to double
	}
	else if ((cmd_type == Tango::DEV_LONG64) || (cmd_type == Tango::DEVVAR_LONG64ARRAY)) 
	{
		*cmd_value >> v_lo64;
		for(vector<Tango::DevLong64>::iterator  it = v_lo64.begin(); it != v_lo64.end(); it++)
			val.push_back((double)(*it));		//convert all to double
	}
	else if (cmd_type == Tango::DEV_STATE)
	{
		//*cmd_value >> v_st;
		//TODO: why extract_read not working with State
		cout << "GG: Do not compile cmd_value->get_type() == Tango::DEV_STATE ???" << endl;
		if(v_st.size() == 0)
		{
			*cmd_value >> stval;
			v_st.push_back(stval);
		}
		for(vector<Tango::DevState>::iterator it = v_st.begin(); it != v_st.end(); it++)
		{
			val.push_back((double)(*it));		//convert all to double
		}
	}
	else if ((cmd_type == Tango::DEV_ULONG) || (cmd_type == Tango::DEVVAR_ULONGARRAY))
	{
		*cmd_value >> v_ulo;
		for(vector<Tango::DevULong>::iterator  it = v_ulo.begin(); it != v_ulo.end(); it++)
			val.push_back((double)(*it));		//convert all to double
	}
	else if ((cmd_type == Tango::DEV_ULONG64) || (cmd_type == Tango::DEVVAR_ULONG64ARRAY)) 
	{
		*cmd_value >> v_ulo64;
		for(vector<Tango::DevULong64>::iterator  it = v_ulo64.begin(); it != v_ulo64.end(); it++)
			val.push_back((double)(*it));		//convert all to double
	}
	else if ((cmd_type == Tango::DEV_DOUBLE) || (cmd_type == Tango::DEVVAR_DOUBLEARRAY)) 
	{
		*cmd_value >> v_do;

		for(vector<Tango::DevDouble>::iterator  it = v_do.begin(); it != v_do.end(); it++)
			val.push_back((double)(*it));		//convert all to double
	}
	else if ((cmd_type == Tango::DEV_FLOAT) || (cmd_type == Tango::DEVVAR_FLOATARRAY)) 
	{
		*cmd_value >> v_fl;
		for(vector<Tango::DevFloat>::iterator  it = v_fl.begin(); it != v_fl.end(); it++)
			val.push_back((double)(*it));		//convert all to double
	}
	else if ((cmd_type == Tango::DEV_BOOLEAN) || (cmd_type == Tango::DEVVAR_BOOLEANARRAY)) 
	{
		//*cmd_value >> v_bo; // GG do not compile ????
		cout << "GG: Do not compile cmd_value->get_type() == Tango::DEV_BOOLEAN ???" << endl;
		for(vector<Tango::DevBoolean>::iterator  it = v_bo.begin(); it != v_bo.end(); it++)
			val.push_back((double)(*it));		//convert all to double
	}
	else if ((cmd_type == Tango::DEV_STRING) || (cmd_type == Tango::DEVVAR_STRINGARRAY)) 
	{
		/*attr_value->extract_read(v_string);
		val_string = *(v_string.begin());*/
		*cmd_value >> val_string;
	}
	else
	{
		ostringstream o;
		o << "unknown type" << ends;
		throw o.str();
	}

}

/*
 * cmd_insert_values
 */
Tango::DeviceData cmd_insert_values(long cmd_type, vector<double> val, vector<string> val_string)
{
	Tango::DevState st;
	vector<Tango::DevState> v_st;
	vector<Tango::DevULong> v_ulo;
	Tango::DevULong ulo;
	vector<Tango::DevULong64> v_ulo64;
	Tango::DevULong64 ulo64;
	vector<Tango::DevUChar> v_uch;
	Tango::DevUChar uch;
	vector<Tango::DevShort> v_sh;
	Tango::DevShort sh;
	vector<Tango::DevUShort> v_ush;
	Tango::DevUShort ush;
	vector<Tango::DevLong> v_lo;
	Tango::DevLong lo;
	vector<Tango::DevLong64> v_lo64;
	Tango::DevLong64 lo64;
	vector<Tango::DevDouble> v_do;
	Tango::DevDouble _do;
	vector<Tango::DevFloat> v_fl;
	Tango::DevFloat fl;
	vector<Tango::DevBoolean> v_bo;
	Tango::DevBoolean bo;
	vector<string> v_string;
	string _string;
	Tango::DeviceData din_value;
	
	if (cmd_type == Tango::DEV_UCHAR)
	{
		if(val.size() > 1)
		{
			for(vector<double>::iterator it = val.begin(); it != val.end(); it++)
				v_uch.push_back((Tango::DevUChar)(*it));		//convert all from double
			//din_value << v_uch;
			cout << "GG: Do not compile cmd_type == Tango::DEV_UCHAR ???" << endl;
		}
		else
		{
			uch = (Tango::DevUChar)(*val.begin());
			// din_value << uch;
			cout << "GG: Do not compile cmd_type == Tango::DEV_UCHAR ???" << endl;
		}
	}
	else if ((cmd_type == Tango::DEV_SHORT) || (cmd_type == Tango::DEVVAR_SHORTARRAY)) 
	{
		if(val.size() > 1)
		{
			for(vector<double>::iterator it = val.begin(); it != val.end(); it++)
				v_sh.push_back((Tango::DevShort)(*it));		//convert all from double
			din_value << v_sh;
		}
		else
		{
			sh = (Tango::DevShort)(*val.begin());
			din_value << sh;
		}
	}
	else if ((cmd_type == Tango::DEV_USHORT) || (cmd_type == Tango::DEVVAR_USHORTARRAY)) 
	{
		if(val.size() > 1)
		{
			for(vector<double>::iterator it = val.begin(); it != val.end(); it++)
				v_ush.push_back((Tango::DevUShort)(*it));		//convert all from double
			din_value << v_ush;
		}
		else
		{
			ush = (Tango::DevUShort)(*val.begin());
			din_value << ush;
		}
	}
	else if ((cmd_type == Tango::DEV_LONG) || (cmd_type == Tango::DEVVAR_LONGARRAY)) 
	{
		if(val.size() > 1)
		{
			for(vector<double>::iterator it = val.begin(); it != val.end(); it++)
				v_lo.push_back((Tango::DevLong)(*it));		//convert all from double
			din_value << v_lo;
		}
		else
		{
			lo = (Tango::DevLong)(*val.begin());
			din_value << lo;
		}
	}
	else if ((cmd_type == Tango::DEV_LONG64) || (cmd_type == Tango::DEVVAR_LONG64ARRAY)) 
	{
		if(val.size() > 1)
		{
			for(vector<double>::iterator it = val.begin(); it != val.end(); it++)
				v_lo64.push_back((Tango::DevLong64)(*it));		//convert all from double
			din_value << v_lo64;
		}
		else
		{
			lo64 = (Tango::DevLong64)(*val.begin());
			din_value << lo64;
		}
	}
	else if (cmd_type == Tango::DEV_STATE)
	{
		if(val.size() > 1)
		{
			for(vector<double>::iterator it = val.begin(); it != val.end(); it++)
				v_st.push_back((Tango::DevState)(*it));		//convert all from double
			//din_value << v_st;
			cout << "GG: Do not compile cmd_type == Tango::DEV_STATE ???" << endl;
		}
		else
		{
			st = (Tango::DevState)(*(val.begin()));
			din_value << st;
		}
	}
	else if ((cmd_type == Tango::DEV_ULONG) || (cmd_type == Tango::DEVVAR_ULONGARRAY)) 
	{
		if(val.size() > 1)
		{
			for(vector<double>::iterator it = val.begin(); it != val.end(); it++)
				v_ulo.push_back((Tango::DevULong)(*it));		//convert all from double
			din_value << v_ulo;
		}
		else
		{
			ulo = (Tango::DevULong)(*(val.begin()));
			din_value << ulo;
		}
	}
	else if ((cmd_type == Tango::DEV_ULONG64) || (cmd_type == Tango::DEVVAR_ULONG64ARRAY)) 
	{
		if(val.size() > 1)
		{
			for(vector<double>::iterator it = val.begin(); it != val.end(); it++)
				v_ulo64.push_back((Tango::DevULong64)(*it));		//convert all from double
			din_value << v_ulo64;
		}
		else
		{
			ulo64 = (Tango::DevULong64)(*(val.begin()));
			din_value << ulo64;
		}
	}
	else if ((cmd_type == Tango::DEV_DOUBLE) || (cmd_type == Tango::DEVVAR_DOUBLEARRAY)) 
	{
		if(val.size() > 1)
		{
			for(vector<double>::iterator it = val.begin(); it != val.end(); it++)
			{
				v_do.push_back((Tango::DevDouble)(*it));		//convert all from double
			}
			din_value << v_do;
		}
		else
		{
			_do = (Tango::DevDouble)(*(val.begin()));
			din_value << _do;
		}
	}
	else if ((cmd_type == Tango::DEV_FLOAT) || (cmd_type == Tango::DEVVAR_FLOATARRAY)) 
	{
		if(val.size() > 1)
		{
			for(vector<double>::iterator it = val.begin(); it != val.end(); it++)
				v_fl.push_back((Tango::DevFloat)(*it));		//convert all from double
			din_value << v_fl;
		}
		else
		{
			fl = (Tango::DevFloat)(*(val.begin()));
			din_value << fl;
		}
	}
	else if ((cmd_type == Tango::DEV_BOOLEAN) || (cmd_type == Tango::DEVVAR_BOOLEANARRAY)) 
	{
		if(val.size() > 1)
		{
			for(vector<double>::iterator it = val.begin(); it != val.end(); it++)
				v_bo.push_back((Tango::DevBoolean)(*it));		//convert all from double
			// din_value << v_bo;
				cout << "GG: Do not compile cmd_type == Tango::DEV_BOOLEAN ???" << endl;
		}
		else
		{
			bo = (Tango::DevBoolean)(*(val.begin()));
			din_value << bo;
		}
	}
	else if ((cmd_type == Tango::DEV_STRING) || (cmd_type == Tango::DEVVAR_STRINGARRAY)) 
	{
		if(val.size() > 1)
		{
			for(vector<string>::iterator it = val_string.begin(); it != val_string.end(); it++)
				v_string.push_back(*it);		//convert all from string
			din_value << v_string;
		}
		else
		{
			_string = *(val_string.begin());
			din_value << _string;
		}
	}
	else
	{
		ostringstream o;
		o << "unknown type" << ends;
		throw o.str();
	}

	return din_value;
}



/*
 * Extract values from DeviceAttribute class (convert numeric data into double)
 */
void extract_values(Tango::DeviceAttribute *attr_value, vector<double> &val, vector<string> &val_string)
{
	Tango::DevState stval;
	vector<Tango::DevState> v_st;
	vector<Tango::DevULong> v_ulo;
	vector<Tango::DevUChar> v_uch;
	vector<Tango::DevShort> v_sh;
	vector<Tango::DevUShort> v_ush;
	vector<Tango::DevLong> v_lo;
	vector<Tango::DevDouble> v_do;
	vector<Tango::DevFloat> v_fl;
	vector<Tango::DevBoolean> v_bo;
	vector<Tango::DevLong64> v_lo64;
	vector<Tango::DevULong64> v_ulo64;
	//vector<string> v_string;

	val.clear();

	if (attr_value->get_type() == Tango::DEV_UCHAR)
	{
		attr_value->extract_read(v_uch);
		for(vector<Tango::DevUChar>::iterator it = v_uch.begin(); it != v_uch.end(); it++)
			val.push_back((double)(*it));		//convert all to double

		try {
			attr_value->extract_set(v_uch);
			for(vector<Tango::DevUChar>::iterator it = v_uch.begin(); it != v_uch.end(); it++)
				val.push_back((double)(*it));		//convert all to double
		}
		catch (Tango::DevFailed &e) {}
	}
	else if (attr_value->get_type() == Tango::DEV_SHORT)
	{
		attr_value->extract_read(v_sh);
		for(vector<Tango::DevShort>::iterator  it = v_sh.begin(); it != v_sh.end(); it++)
			val.push_back((double)(*it));		//convert all to double

		try {
			attr_value->extract_set(v_sh);
			for(vector<Tango::DevShort>::iterator  it = v_sh.begin(); it != v_sh.end(); it++)
				val.push_back((double)(*it));		//convert all to double
		}
		catch (Tango::DevFailed &e) {}

	}
	else if (attr_value->get_type() == Tango::DEV_USHORT)
	{
		attr_value->extract_read(v_ush);
		for(vector<Tango::DevUShort>::iterator  it = v_ush.begin(); it != v_ush.end(); it++)
			val.push_back((double)(*it));		//convert all to double

		try {
			attr_value->extract_set(v_ush);
			for(vector<Tango::DevUShort>::iterator  it = v_ush.begin(); it != v_ush.end(); it++)
				val.push_back((double)(*it));		//convert all to double
		}
		catch (Tango::DevFailed &e) {}

	}
	else if (attr_value->get_type() == Tango::DEV_LONG)
	{
		attr_value->extract_read(v_lo);
		for(vector<Tango::DevLong>::iterator  it = v_lo.begin(); it != v_lo.end(); it++)
			val.push_back((double)(*it));		//convert all to double

		try {
			attr_value->extract_set(v_lo);
			for(vector<Tango::DevLong>::iterator  it = v_lo.begin(); it != v_lo.end(); it++)
				val.push_back((double)(*it));		//convert all to double
		}
		catch (Tango::DevFailed &e) {}

	}
	else if (attr_value->get_type() == Tango::DEV_LONG64)
	{
		attr_value->extract_read(v_lo64);
		for(vector<Tango::DevLong64>::iterator  it = v_lo64.begin(); it != v_lo64.end(); it++)
			val.push_back((double)(*it));		//convert all to double

		try {
			attr_value->extract_set(v_lo64);
			for(vector<Tango::DevLong64>::iterator  it = v_lo64.begin(); it != v_lo64.end(); it++)
				val.push_back((double)(*it));		//convert all to double
		}
		catch (Tango::DevFailed &e) {}

	}
	else if (attr_value->get_type() == Tango::DEV_STATE)
	{
		attr_value->extract_read(v_st);
		//TODO: why extract_read not working with State
		if(v_st.size() == 0)
		{
			*(attr_value) >> stval;
			v_st.push_back(stval);
		}
		for(vector<Tango::DevState>::iterator it = v_st.begin(); it != v_st.end(); it++)
		{
			val.push_back((double)(*it));		//convert all to double
		}
	}
	else if (attr_value->get_type() == Tango::DEV_ULONG)
	{
		attr_value->extract_read(v_ulo);
		for(vector<Tango::DevULong>::iterator  it = v_ulo.begin(); it != v_ulo.end(); it++)
			val.push_back((double)(*it));		//convert all to double

		try {
			attr_value->extract_set(v_ulo);
			for(vector<Tango::DevULong>::iterator  it = v_ulo.begin(); it != v_ulo.end(); it++)
				val.push_back((double)(*it));		//convert all to double
		}
		catch (Tango::DevFailed &e) {}

	}
	else if (attr_value->get_type() == Tango::DEV_ULONG64)
	{
		attr_value->extract_read(v_ulo64);
		for(vector<Tango::DevULong64>::iterator  it = v_ulo64.begin(); it != v_ulo64.end(); it++)
			val.push_back((double)(*it));		//convert all to double

		try {
			attr_value->extract_set(v_ulo64);
			for(vector<Tango::DevULong64>::iterator  it = v_ulo64.begin(); it != v_ulo64.end(); it++)
				val.push_back((double)(*it));		//convert all to double
		}
		catch (Tango::DevFailed &e) {}

	}
	else if (attr_value->get_type() == Tango::DEV_DOUBLE)
	{
		attr_value->extract_read(v_do);

		for(vector<Tango::DevDouble>::iterator  it = v_do.begin(); it != v_do.end(); it++)
			val.push_back((double)(*it));		//convert all to double

		try {
			attr_value->extract_set(v_do);
			for(vector<Tango::DevDouble>::iterator  it = v_do.begin(); it != v_do.end(); it++)
				val.push_back((double)(*it));		//convert all to double
		}
		catch (Tango::DevFailed &e) {}

	}
	else if (attr_value->get_type() == Tango::DEV_FLOAT)
	{
		attr_value->extract_read(v_fl);
		for(vector<Tango::DevFloat>::iterator  it = v_fl.begin(); it != v_fl.end(); it++)
			val.push_back((double)(*it));		//convert all to double

		try {
			attr_value->extract_set(v_fl);
			for(vector<Tango::DevFloat>::iterator  it = v_fl.begin(); it != v_fl.end(); it++)
			val.push_back((double)(*it));		//convert all to double
		}
		catch (Tango::DevFailed &e) {}

	}
	else if (attr_value->get_type() == Tango::DEV_BOOLEAN)
	{
		attr_value->extract_read(v_bo);
		for(vector<Tango::DevBoolean>::iterator  it = v_bo.begin(); it != v_bo.end(); it++)
			val.push_back((double)(*it));		//convert all to double

		try {
			attr_value->extract_set(v_bo);
			for(vector<Tango::DevBoolean>::iterator  it = v_bo.begin(); it != v_bo.end(); it++)
				val.push_back((double)(*it));		//convert all to double
		}
		catch (Tango::DevFailed &e) {}

	}
	else if (attr_value->get_type() == Tango::DEV_STRING)
	{
		/*attr_value->extract_read(v_string);
		val_string = *(v_string.begin());*/
		attr_value->extract_read(val_string);
	}
	else
	{
		ostringstream o;
		o << "unknown type=" << (int)attr_value->get_type() << ends;
		throw o.str();
	}

}


Tango::DeviceAttribute* insert_values(Tango::AttributeInfoEx attr_info, string attr_name, vector<double> val, vector<string> val_string)
{
	int i=0;
	Tango::DevState st;
	vector<Tango::DevState> v_st;
	vector<Tango::DevULong> v_ulo;
	Tango::DevULong ulo;
	vector<Tango::DevULong64> v_ulo64;
	Tango::DevULong64 ulo64;
	vector<Tango::DevUChar> v_uch;
	Tango::DevUChar uch;
	vector<Tango::DevShort> v_sh;
	Tango::DevShort sh;
	vector<Tango::DevUShort> v_ush;
	Tango::DevUShort ush;
	vector<Tango::DevLong> v_lo;
	Tango::DevLong lo;
	vector<Tango::DevLong64> v_lo64;
	Tango::DevLong64 lo64;
	vector<Tango::DevDouble> v_do;
	Tango::DevDouble _do;
	vector<Tango::DevFloat> v_fl;
	Tango::DevFloat fl;
	vector<Tango::DevBoolean> v_bo;
	Tango::DevBoolean bo;
	vector<string> v_string;
	string _string;
	Tango::DeviceAttribute *din_value;

	if (attr_info.data_type == Tango::DEV_UCHAR)
	{
		if(attr_info.max_dim_x > 1)
		{
			for(vector<double>::iterator it = val.begin(); it != val.end() && i < attr_info.max_dim_x; it++, i++)
				v_uch.push_back((Tango::DevUChar)(*it));		//convert all from double
			din_value = new Tango::DeviceAttribute(attr_name,v_uch);
		}
		else
		{
			uch = (Tango::DevUChar)(*val.begin());
			din_value = new Tango::DeviceAttribute(attr_name,uch);
		}
	}
	else if (attr_info.data_type == Tango::DEV_SHORT)
	{
		if(attr_info.max_dim_x > 1)
		{
			for(vector<double>::iterator it = val.begin(); it != val.end() && i < attr_info.max_dim_x; it++, i++)
				v_sh.push_back((Tango::DevShort)(*it));		//convert all from double
			din_value = new Tango::DeviceAttribute(attr_name,v_sh);
		}
		else
		{
			sh = (Tango::DevShort)(*val.begin());
			din_value = new Tango::DeviceAttribute(attr_name,sh);
		}
	}
	else if (attr_info.data_type == Tango::DEV_USHORT)
	{
		if(attr_info.max_dim_x > 1)
		{
			for(vector<double>::iterator it = val.begin(); it != val.end() && i < attr_info.max_dim_x; it++, i++)
				v_ush.push_back((Tango::DevUShort)(*it));		//convert all from double
			din_value = new Tango::DeviceAttribute(attr_name,v_ush);
		}
		else
		{
			ush = (Tango::DevUShort)(*val.begin());
			din_value = new Tango::DeviceAttribute(attr_name,ush);
		}
	}
	else if (attr_info.data_type == Tango::DEV_LONG)
	{
		if(attr_info.max_dim_x > 1)
		{
			for(vector<double>::iterator it = val.begin(); it != val.end() && i < attr_info.max_dim_x; it++, i++)
				v_lo.push_back((Tango::DevLong)(*it));		//convert all from double
			din_value = new Tango::DeviceAttribute(attr_name,v_lo);
		}
		else
		{
			lo = (Tango::DevLong)(*val.begin());
			din_value = new Tango::DeviceAttribute(attr_name,lo);
		}
	}
	else if (attr_info.data_type == Tango::DEV_LONG64)
	{
		if(attr_info.max_dim_x > 1)
		{
			for(vector<double>::iterator it = val.begin(); it != val.end() && i < attr_info.max_dim_x; it++, i++)
				v_lo64.push_back((Tango::DevLong64)(*it));		//convert all from double
			din_value = new Tango::DeviceAttribute(attr_name,v_lo64);
		}
		else
		{
			lo64 = (Tango::DevLong64)(*val.begin());
			din_value = new Tango::DeviceAttribute(attr_name,lo64);
		}
	}
	else if (attr_info.data_type == Tango::DEV_STATE)
	{
		if(attr_info.max_dim_x > 1)
		{
			for(vector<double>::iterator it = val.begin(); it != val.end() && i < attr_info.max_dim_x; it++, i++)
				v_st.push_back((Tango::DevState)(*it));		//convert all from double
			din_value = new Tango::DeviceAttribute(attr_name,v_st);
		}
		else
		{
			st = (Tango::DevState)(*(val.begin()));
			din_value = new Tango::DeviceAttribute(attr_name,st);
		}
	}
	else if (attr_info.data_type == Tango::DEV_ULONG)
	{
		if(attr_info.max_dim_x > 1)
		{
			for(vector<double>::iterator it = val.begin(); it != val.end() && i < attr_info.max_dim_x; it++, i++)
				v_ulo.push_back((Tango::DevULong)(*it));		//convert all from double
			din_value = new Tango::DeviceAttribute(attr_name,v_ulo);
		}
		else
		{
			ulo = (Tango::DevULong)(*(val.begin()));
			din_value = new Tango::DeviceAttribute(attr_name,ulo);
		}
	}
	else if (attr_info.data_type == Tango::DEV_ULONG64)
	{
		if(attr_info.max_dim_x > 1)
		{
			for(vector<double>::iterator it = val.begin(); it != val.end() && i < attr_info.max_dim_x; it++, i++)
				v_ulo64.push_back((Tango::DevULong64)(*it));		//convert all from double
			din_value = new Tango::DeviceAttribute(attr_name,v_ulo64);
		}
		else
		{
			ulo64 = (Tango::DevULong64)(*(val.begin()));
			din_value = new Tango::DeviceAttribute(attr_name,ulo64);
		}
	}
	else if (attr_info.data_type == Tango::DEV_DOUBLE)
	{
		if(attr_info.max_dim_x > 1)
		{
			for(vector<double>::iterator it = val.begin(); it != val.end() && i < attr_info.max_dim_x; it++, i++)
			{
				v_do.push_back((Tango::DevDouble)(*it));		//convert all from double
			}
			din_value = new Tango::DeviceAttribute(attr_name,v_do);
		}
		else
		{
			_do = (Tango::DevDouble)(*(val.begin()));
			din_value = new Tango::DeviceAttribute(attr_name,_do);
		}
	}
	else if (attr_info.data_type == Tango::DEV_FLOAT)
	{
		if(attr_info.max_dim_x > 1)
		{
			for(vector<double>::iterator it = val.begin(); it != val.end() && i < attr_info.max_dim_x; it++, i++)
				v_fl.push_back((Tango::DevFloat)(*it));		//convert all from double
			din_value = new Tango::DeviceAttribute(attr_name,v_fl);
		}
		else
		{
			fl = (Tango::DevFloat)(*(val.begin()));
			din_value = new Tango::DeviceAttribute(attr_name,fl);
		}
	}
	else if (attr_info.data_type == Tango::DEV_BOOLEAN)
	{
		if(attr_info.max_dim_x > 1)
		{
			for(vector<double>::iterator it = val.begin(); it != val.end() && i < attr_info.max_dim_x; it++, i++)
				v_bo.push_back((Tango::DevBoolean)(*it));		//convert all from double
			din_value = new Tango::DeviceAttribute(attr_name,v_bo);
		}
		else
		{
			bo = (Tango::DevBoolean)(*(val.begin()));
			din_value = new Tango::DeviceAttribute(attr_name,bo);
		}
	}
	else if (attr_info.data_type == Tango::DEV_STRING)
	{
		if(attr_info.max_dim_x > 1)
		{
			for(vector<string>::iterator it = val_string.begin(); it != val_string.end() && i < attr_info.max_dim_x; it++, i++)
				v_string.push_back(*it);		//convert all from string
			din_value = new Tango::DeviceAttribute(attr_name,v_string);
		}
		else
		{
			_string = *(val_string.begin());
			din_value = new Tango::DeviceAttribute(attr_name,_string);
		}
	}
	else
	{
		ostringstream o;
		o << "unknown type=" << (int)attr_info.data_type << ends;
		throw o.str();
	}

	return din_value;
}





























