/*
 * util.h
 * Common function between feedback, actuator and sensor classes
 */

#ifndef UTIL_H
#define UTIL_H

#include <tango.h>
#include <algorithm>
#include <numeric>      // std::accumulate
#include <deque>
#include <math.h>
#include <vector>
#include <iostream>
#include <stdio.h>
#include <string.h>


#define MIMOFB_FILT_NONE 0
#define MIMOFB_FILT_MEAN 1
#define MIMOFB_FILT_MEDIAN 2
#define MIMOFB_FILT_STD  3

#define MIMOFB_LOGLEVEL_NONE 0
#define MIMOFB_LOGLEVEL_INFO 1
#define MIMOFB_LOGLEVEL_DEBUG 2
#define MIMOFB_LOG_SIZE 4096
#define MIMOFB_LOG_DATA_SIZE 100000

#define MIMOFB_LOG_MODE_SAMPLES 0
#define MIMOFB_LOG_MODE_TIME 1
#define MIMOFB_LOG_MODE_TIMESPAN 2

#define MIMOFB_LOG_MODE_SAMPLES_TSCALE 3
#define MIMOFB_LOG_MODE_TIME_TSCALE 4
#define MIMOFB_LOG_MODE_TIMESPAN_TSCALE 5

using namespace std;


/*
 * map which converts a string into a tango state
 */
static std::map<std::string, Tango::DevState> tango_states = {
 	{"ON", Tango::ON},
 	{"OFF", Tango::OFF},
 	{"CLOSE", Tango::CLOSE},
 	{"OPEN", Tango::OPEN},
 	{"INSERT", Tango::INSERT},
 	{"EXTRACT", Tango::EXTRACT},
 	{"MOVING", Tango::MOVING},
 	{"STANDBY", Tango::STANDBY},
 	{"FAULT", Tango::FAULT},
 	{"INIT", Tango::INIT},
 	{"RUNNING", Tango::RUNNING},
 	{"ALARM", Tango::ALARM},
 	{"DISABLE", Tango::DISABLE},
 	{"UNKNOWN", Tango::UNKNOWN},
 	{"", Tango::UNKNOWN}
 };


/*
 * function which split string (once defined a separator 
 */
extern void string_explode(string str, string separator, vector<string>* results);

/*
 * sleep time in msec
 */
extern int local_msleep(long msec);

/*
 * process_data
 */
extern double low_pass_filt_data(vector <double> values, int32_t process_mode);

/*
 * check_limits
 */
extern void check_limits(double value, double min_value, double max_value);


/*
 * out_of_range_filt_data
 */
extern vector <double> out_of_range_filt_data(vector <double> values, double min_value, double max_value, double max_tol, bool zero_error, bool zero_skip);

/*
 * check tango states
 */
extern Tango::DevState check_state(vector <Tango::DevState> &allowed_states, Tango::DeviceProxy *device, long max_retry, long retry_sleep);


extern void cmd_extract_values(Tango::DeviceData *cmd_value, vector<double> &val, vector<string> &val_string);

/*
 * cmd_insert_values
 */
extern Tango::DeviceData cmd_insert_values(long cmd_type, vector<double> val, vector<string> val_string);

/*
 * Extract values from DeviceAttribute class (convert numeric data into double)
 */
extern void extract_values(Tango::DeviceAttribute *attr_value, vector<double> &val, vector<string> &val_string);

/*
 * Tango::DeviceAttribute* insert_values
 */
extern Tango::DeviceAttribute* insert_values(Tango::AttributeInfoEx attr_info, string attr_name, vector<double> val, vector<string> val_string);

/*
 * string_explode
 */
extern void string_explode(string str, string separator, vector<string>* results);

/*
 * generate a random uniform distributed number between 0 and 1
 */
extern double random_uniform_generator();

/*
 * generate a gaussian distributed number with mean 0 and std equel to 1
 */
extern double random_gaussian_generator();


extern vector<int32_t> shuffled_vector_index(int32_t num_elem);

/*
 * log_buffer template class
 */
template <class T>
  class log_buffer {
  private:
    std::deque <T> log_hist_b;
    int32_t max_num_elem;
    int32_t log_level;
    std::deque <double> log_hist_time;
  public:
    ~log_buffer();
    void add_log(const char *, int32_t level);
    void add_log(T const&);
    std::vector <T> get_log_hist(int32_t num_values, bool invert);    
    std::vector <double> get_log_hist(double num_values, bool invert);
    std::vector <double> get_log_hist_time(double last_sec, bool invert);
    std::vector <double> get_log_hist_timespan(double start_time, double end_time, bool invert);    
    void set_log_level(int32_t log_lv) {log_level = log_lv;}
    int32_t get_log_level(void) { return log_level;}
    void set_size(int32_t num_elem) {max_num_elem = num_elem;}
    int32_t get_size(void) { return max_num_elem;}
    
};


template <class T> void log_buffer<T>::add_log(const char *data_in, int32_t level)
{

	if (level > log_level)
		return;

  time_t tt = time(0); 
  char timebuf[500];
  
  strftime(timebuf, 24, "%Y-%m-%d-%H:%M:%S: ", localtime(&tt));

  //if size is too long remove oldest element
  if ((int32_t)log_hist_b.size() >= max_num_elem)
    log_hist_b.pop_back();

  std::string localmsg(strncat(timebuf, data_in, 470));
  log_hist_b.push_front(localmsg);
  
}

template <class T> void log_buffer<T>::add_log(T const& data)
{

  //if size is too long remove oldest element
  if ((int32_t)log_hist_b.size() >= max_num_elem) {
    log_hist_b.pop_back();
    log_hist_time.pop_back();
  }

  log_hist_b.push_front(data);
  
  timespec tms;
  clock_gettime(CLOCK_REALTIME,&tms);
  // microsec resolution
  log_hist_time.push_front((double)tms.tv_sec+(double)tms.tv_nsec/1000000000);

}

template <class T> std::vector<T> log_buffer<T>::get_log_hist(int32_t num_values, bool invert)
{
	std::vector <T> values;
	int cnt = 0;
	for (typename std::deque<T>::iterator it = log_hist_b.begin(); it != log_hist_b.end(); ++it) {
		if (invert == false) 
		  values.push_back((*it));
		else
		  values.insert(values.begin(),(*it));
		cnt++;
		if ((cnt >= max_num_elem) || (cnt >= num_values))
			break;
	}
	return values;	

}

template <class T> std::vector<double> log_buffer<T>::get_log_hist(double num_values, bool invert)
{
	std::vector <T> values;
	int cnt = 0;
	
	for (std::pair<typename std::deque<T>::iterator, typename std::deque<double>::iterator> it(log_hist_b.begin(), log_hist_time.begin());
    		(it.first != log_hist_b.end()) && (it.second != log_hist_time.end()) ;
     		++it.first, ++it.second) {

		if (invert == false) {
			values.push_back((double)(*it.first));
			values.push_back(*it.second);
		}				
		else {
			values.insert(values.begin(),(*it.second));    		
			values.insert(values.begin(),(double)(*it.first)); 				    		
		}

		cnt++;
		
		if ((cnt >= max_num_elem) || (cnt >= num_values)) 
			break;

	}	

	return values;	

}


template <class T> std::vector<double> log_buffer<T>::get_log_hist_time(double last_sec, bool invert)
{
	std::vector <T> values;
	int cnt = 0;
	
	timespec tms;
	clock_gettime(CLOCK_REALTIME,&tms);
	double curr_time = (double)tms.tv_sec + (double)tms.tv_nsec/1000000000;

	for (std::pair<typename std::deque<T>::iterator, typename std::deque<double>::iterator> it(log_hist_b.begin(), log_hist_time.begin());
    		(it.first != log_hist_b.end()) && (it.second != log_hist_time.end()) ;
     		++it.first, ++it.second) {

    		if (((*it.second) + last_sec) > curr_time) {
    		
			if (invert == false) {
				values.push_back((double)(*it.first));
				values.push_back(*it.second);
			}				
			else {
				values.insert(values.begin(),(*it.second));    		
				values.insert(values.begin(),(double)(*it.first)); 				    		
    			}

			cnt++;
			
			if (cnt >= max_num_elem) 
				break;
		}	

	}	
		
	return values;	

}

template <class T> std::vector<double> log_buffer<T>::get_log_hist_timespan(double start_time, double end_time, bool invert)
{
	std::vector <T> values;
	int cnt = 0;
	
	timespec tms;
	clock_gettime(CLOCK_REALTIME,&tms);
	double curr_time = (double)tms.tv_sec+(double)tms.tv_nsec/1000000000;

	for (std::pair<typename std::deque<T>::iterator, typename std::deque<double>::iterator> it(log_hist_b.begin(), log_hist_time.begin());
    		(it.first != log_hist_b.end()) && (it.second != log_hist_time.end()) ;
     		++it.first, ++it.second) {
    	
    		if (((*it.second) >= start_time) && ((*it.second) <= end_time)) {
    		
			if (invert == false) {
				values.push_back((double)(*it.first));
				values.push_back(*it.second);
			}				
			else {
				values.insert(values.begin(),(*it.second));    		
				values.insert(values.begin(),(double)(*it.first)); 				    		
    			}

			cnt++;
			
			if (cnt >= max_num_elem) 
				break;
		}	

	}	
		
	return values;	

}

template <class T> log_buffer<T>::~log_buffer()
{
  log_hist_b.clear();
}


/**
*	compares two vector lengths
*/
template<class T>
void comp(vector<T> v1, vector<T> v2){
	if(v1.size() != v2.size()){
		cout << "vectors not the same size\n";
		exit(1);
	}
}

/**
*	Euclidean distance between two vectors of type T such that T has binary +,-,* 
*/
template<class T>
double euclidean(vector<T> v1, vector<T> v2){
	comp(v1, v2);
	T diff, sum;

	diff = v1[0] - v2[0];
	sum = diff * diff;

	for (unsigned int i=1; i < v1.size(); i++){
		diff = v1[i] - v2[i];
		sum += diff * diff;
	}
	return sqrt(double(sum));
}

/**
*	Jaccard Coefficient.	Use for asymetric binary values
*/
template<class T>
double jaccard(vector<T> v1, vector<T> v2){
	comp(v1, v2);
	int f11 = 0, f00 = 0;

	for (unsigned int i=0; i < v1.size(); i++){
		if(v1[i] == v2[i]){
			if(v1[i])
				f11++;
			else
				f00++;
		}
	}
	return double(f11) / double(v1.size() - (f11+f00));
}

/**
*	Cosine
*/
template<class T>
double cosine(vector<T> v1, vector<T> v2){
	comp(v1, v2);

	T lv1 = v1[0] * v1[0];
	T lv2 = v2[0] * v2[0];
	T dot12 = v1[0] * v2[0];

	for (unsigned int i=0; i < v1.size(); i++){
		lv1 += v1[i] * v1[i];
		lv2 += v2[i] * v2[i];
		dot12 += v1[i] * v2[i];
	}
	return double(dot12) / ( sqrt(double(lv1)) * sqrt(double(lv2)) );
}

/**
*	The mean of a vector
*/
template<class T>
double mean(vector<T> v1){
	T sum = v1[0];
	for (unsigned int i=1; i < v1.size(); i++)
		sum += v1[i];
	return double(sum) / double(v1.size());
}

/**
*	The Covariance
*/
template<class T>
double covariance(vector<T> v1, vector<T> v2){
	comp(v1, v2);
	double mean1 = mean(v1), mean2 = mean(v2);
	double sum = (double(v1[0]) - mean1) * (double(v2[0]) - mean2);
	for (unsigned int i=1; i < v1.size(); i++){
		sum += (double(v1[i]) - mean1) * (double(v2[i]) - mean2);
	}
	return double(sum) / double(v1.size()-1);
}

/**
*	standard deviation the covariance where both vectors are the same.
*/
template<class T>
double std_dev(vector<T> v1){
	return sqrt(covariance(v1, v1));
}

/**
*	Pearson Correlation
*/
template<class T>
double pearson(vector<T> v1, vector<T> v2){
	if (std_dev(v1) * std_dev(v2) == 0){
		cout << "( a standard deviaton was 0 )";
		return -2; // I dont know what to do here???
	}
	return covariance(v1,v2) / ( std_dev(v1) * std_dev(v2));
}



#endif
